﻿<#
.Synopsis
    Moves HPC nodes to a target HPC Pack cluster.

.DESCRIPTION
    This script moves HPC nodes that meet the specified criteria to a target HPC Pack cluster. 

.NOTES
    Wildcard values for the ‘Name’ and ‘TemplateName’ parameters are not supported when 'NoSourceCluster' is specified
    Windows PowerShell remoting must be enabled in the HPC nodes when 'NoSourceCluster' is specified 

.EXAMPLE
    PS > Move-HpcNode.ps1 -Name HpcNode-* -TargetCluster "NewHN1,NewHN2,NewHN3"
    Moves HPC nodes named "HpcNode-*" from the current HPC cluster to a new HPC cluster with cluster connection string "NewHN1,NewHN2,NewHN3"
       
.EXAMPLE
    PS > Move-HpcNode.ps1 -Name HpcNode-1,HpcNode-2 -TargetCluster "NewHN1,NewHN2,NewHN3" -SourceCluster "OldHN1,OldHN2,OldHN3"
    Moves HPC nodes "HpcNode-1" and "HpcNode-2" from the HPC cluster "MyOldCluster" to HPC cluster "NewHN1,NewHN2,NewHN3"
       
.EXAMPLE
    PS > Move-HpcNode.ps1 -TemplateName NodeTemplate-* -TargetCluster MyNewCluster
    Moves all HPC nodes associated with HPC node template "NodeTemplate-*" to HPC cluster "MyNewCluster"

.EXAMPLE
    PS > Move-HpcNode.ps1 -Name HpcNode-1,HpcNode-2 -TargetCluster "NewHN1,NewHN2,NewHN3" -NoSourceCluster
    Moves HPC nodes "HpcNode-1" and "HpcNode-2" which don't belong to any HPC cluster currently to HPC cluster "NewHN1,NewHN2,NewHN3"
#>
 
[CmdletBinding(ConfirmImpact="Medium", DefaultParametersetName="Node", SupportsShouldProcess=$true)]
Param
(
    # A list of one or more names for the nodes that you want to move.
    [Parameter(Mandatory=$true, Position=0, ParameterSetName="Node")]
    [ValidateNotNullOrEmpty()]
    [String[]] $Name,

    # A list of names for the node templates that are associated with the nodes that you want to move. You cannot specify this parameter if 'NoSourceCluster' is specified.
    [Parameter(Mandatory=$true, Position=0, ParameterSetName="Template")]
    [ValidateNotNullOrEmpty()]
    [String[]] $TemplateName,

    # The target HPC Pack cluster connection string that you want the nodes moved to, for example "NewHN1,NewHN2,NewHN3".
    [Parameter(Mandatory=$true, Position=1)]
    [ValidateNotNullOrEmpty()]
    [String] $TargetCluster,

    # If specified, identifies the cluster connection string of the HPC cluster that currently includes the nodes. If you specify neither the 'SourceCluster' nor the 'NoSourceCluster' parameter, this cmdlet uses the CCP_SCHEDULER environment variable as the source HPC cluster connection string.
    [Parameter(Mandatory=$false)]
    [ValidateNotNullOrEmpty()]
    [String] $SourceCluster,

    # If specified, indicates that no source HPC cluster is available (for example, the head node of the source cluster was destroyed).
    [Parameter(Mandatory=$false, ParameterSetName="Node")]
    [ValidateNotNullOrEmpty()]
    [Switch] $NoSourceCluster,

    # If specified, move the HPC nodes without confirmation prompt. 
    [Parameter(Mandatory=$false)]
    [Switch] $Force
)

$ErrorActionPreference = "Stop"
$VerbosePreference = "Continue"
$SetClusterScriptFile = "Set-HpcClusterName.ps1"
$StartTimestr = Get-Date -Format "MM_dd-HH_mm_ss"
$noHN = $NoSourceCluster.IsPresent
if(-not $noHN)
{
    if([String]::IsNullOrEmpty($SourceCluster))
    {
        $SourceCluster = $env:CCP_SCHEDULER
        if([String]::IsNullOrEmpty($SourceCluster))
        {
            Write-Error "You must use either 'SourceCluster' to specify the current cluster which the HPC cluster nodes belong to, or 'NoSourceCluster' if no current HPC cluster is available."
            return
        }
    }

    Add-PSSnapIn Microsoft.HPC
    if(-not $?)
    {
        throw "Failed to Load Microsoft.HPC SnapIn"
    }

    if ($PSCmdlet.ParameterSetName -eq "Node")
    {
        $hpcNodes = Get-HpcNode -Name $Name -Scheduler $SourceCluster -ErrorAction SilentlyContinue
    }
    else
    {
        foreach($tempName in $TemplateName)
        {
            if("HeadNode Template" -like $TemplateName)
            {
                Write-Warning "Cannot move an HPC head node."
            }
        }

        $hpcNodes = Get-HpcNode -TemplateName $TemplateName -Scheduler $SourceCluster -ErrorAction SilentlyContinue
    }

    if($hpcNodes -eq $null)
    {
        throw "No HPC cluster nodes can be moved."
    }

    $canMoveNodes = @()
    foreach($node in $hpcNodes)
    {
        if($node.IsHeadNode)
        {
            Write-Warning "$($node.NetbiosName) cannot be moved because it is a head node"
        }
        elseif(($node.NodeRole -band [Microsoft.ComputeCluster.CCPPSH.NodeCommand+NodeRole]::AzureWorkerNode) -gt 0)
        {
            Write-Warning "$($node.NetbiosName) cannot be moved because it is an Azure node"            
        }
        elseif(($node.NodeState -ne "Online") -and ($node.NodeState -ne "Offline") -and ($node.NodeState -ne "Draining"))
        {
            Write-Warning "$($node.NetbiosName) cannot be moved because it is in $($node.NodeState) state"
        }
        elseif($node.HealthState -eq "Error")
        {
            Write-Warning "$($node.NetbiosName) cannot be moved because it is unreachable"
        }
        else
        {
            $canMoveNodes += $node
        }
    }

    if($canMoveNodes.Count -eq 0)
    {
        throw "No HPC cluster nodes can be moved."
    }

    if(-not $Force.IsPresent)
    {
        $canMoveNodeNames = $canMoveNodes.NetbiosName -join ","
        if(-not $PsCmdlet.ShouldProcess("Move the HPC cluster nodes to another HPC cluster, the operation cannot be reverted",
                                        "The following HPC cluster nodes will be moved to the target HPC cluster $TargetCluster :`n $canMoveNodeNames`nDo you want to continue?",
                                        "Move HPC cluster nodes to HPC cluster $TargetCluster"))
        {
            return
        }
    }

    if($Force.IsPresent)
    {
        Set-HpcNodeState -Node $canMoveNodes -State offline -Scheduler $SourceCluster -Force -ErrorAction Continue | Out-Null
    }
    else
    {
        Set-HpcNodeState -Node $canMoveNodes -State offline -Scheduler $SourceCluster -ErrorAction Continue | Out-Null
    }

    $reminst = (Get-HpcClusterRegistry -Scheduler $SourceCluster -PropertyName InstallShare).Value
    # Use Clusrun.exe to schedule task on the selected HPC cluster nodes to update Cluster connection string, the HPC Services in the these nodes will eventually be restarted to make the Cluster connection string setting effective
    # We give 15 seconds delay so that the command can return before HPC Services restarted on these nodes.
    $tempScriptFile = "$reminst\Setup\Set-HpcClusterName-$StartTimestr.ps1"
    Copy-Item $PSScriptRoot\$SetClusterScriptFile $tempScriptFile -Force
    $clusrunParam = '/nodes:"{0}" /scheduler:{1} PowerShell.exe -ExecutionPolicy ByPass -Command "Copy-Item ''{2}'' $env:TEMP\Set-HpcClusterName.ps1 -Force; & $env:TEMP\Set-HpcClusterName.ps1 -ConnectionString {3} -Delay 15 -RunAsScheduledTask"' -f ($canMoveNodes.NetBiosName -join ","), $SourceCluster, $tempScriptFile, $TargetCluster
    Start-Process -FilePath "clusrun.exe" -ArgumentList $clusrunParam -Wait -NoNewWindow
    Remove-Item $tempScriptFile -Force -ErrorAction SilentlyContinue
    Remove-HpcNode -Node $canMoveNodes -Scheduler $SourceCluster -Confirm:$false
}
else
{
    if(-not [String]::IsNullOrEmpty($SourceCluster))
    {
        throw "You cannot specifiy both 'SourceCluster' and 'NoSourceCluster'"    
    }

    $wildcardNames = $Name | ?{$_ -match '[*, ?, \[, \]]'}
    if($wildcardNames -ne $null)
    {
        throw ("A wildCard Name is not supported when 'NoSourceCluster' is specified: " + ($wildcardNames -join ","))
    }

    if(-not $Force.IsPresent)
    {
        $tryMoveNames = $Name -join ","
        if(-not $PsCmdlet.ShouldProcess("Move the HPC cluster nodes to another HPC cluster, the operation cannot be reverted",
                                        "The following HPC cluster nodes will be moved to the target HPC cluster $TargetCluster :`n $tryMoveNames`nDo you want to continue?",
                                        "Move HPC cluster nodes to HPC cluster $TargetCluster"))
        {
            return
        }
    }

    $ThrottleLimit = 20
    $MaxRetry = 3
    $FailedNodes = @()

    $retryTable = @{}
    $waitingList = New-Object Collections.Generic.List[String]
    $Name | %{$waitingList.Add($_); $retryTable[$_]=0}
    $movingJobs = @()

    $scriptFilePath = "$PSScriptRoot\$SetClusterScriptFile"
    if(-not (Test-Path -Path $scriptFilePath -PathType Leaf))
    {
        $scriptFilePath = [System.IO.Path]::Combine($env:CCP_HOME, "Bin\$SetClusterScriptFile")
    }
    
    $scriptContent = $null
    if(Test-Path -Path $scriptFilePath -PathType Leaf)
    {
        $scriptContent = [IO.File]::ReadAllBytes($scriptFilePath)
    }
    else
    {
        throw "$SetClusterScriptFile not found"
    }

    while(($waitingList.Count -gt 0) -or ($movingJobs.Count -gt 0))
    {
        $runningJobs = @()
        foreach($job in $movingJobs)
        {
            $nodeName = $job.Name
            if($job.State -eq "Completed")
            {
                Write-Verbose "Successfully moved $nodeName to $TargetCluster"
                Remove-Job -Job $job
            }
            elseif($job.State -eq "Failed")
            {
                $failedReason = $job.ChildJobs[0].JobStateInfo.Reason
                if($failedReason -is [System.Management.Automation.Remoting.PSRemotingTransportException])
                {
                    if($retryTable[$nodeName] -lt $MaxRetry)
                    {
                        Write-Verbose "Failed to move $nodeName to $TargetCluster, retry later ..."
                        $waitingList.Add($nodeName)
                        $retryTable[$nodeName] += 1
                        Remove-Job -Job $job
                        continue
                    }
                }

                Write-Warning "Failed to move $nodeName to $TargetCluster : $failedReason"
                $FailedNodes += $nodeName
                Remove-Job -Job $job
            }
            else
            {
                $runningJobs += $job
            }
        }

        $movingJobs = $runningJobs

        $curJobCount = $movingJobs.Count
        if($curJobCount -ge $ThrottleLimit)
        {
            Start-Sleep -Seconds 2
            continue
        }
        
        $leftCapacity = $ThrottleLimit - $curJobCount
        while(($leftCapacity -gt 0) -and ($waitingList.Count -gt 0))
        {
            $nodeName = $waitingList[0]
            $waitingList.RemoveAt(0) | Out-Null
            Write-Verbose "Moving HPC Node $nodeName ..."
            $movingJobs += Invoke-Command -ComputerName $nodeName -ScriptBlock {
                param([String]$clusName, [Byte[]]$scriptContent)
                
                $SetClusterScriptFile = "$env:temp\Set-HpcClusterName.ps1"
                if(Test-Path -Path $SetClusterScriptFile -PathType Leaf)
                {
                    Remove-Item $SetClusterScriptFile -Force
                }

                try
                {
                    [IO.File]::WriteAllBytes($SetClusterScriptFile, $scriptContent)
                }
                catch
                {
                    throw "Failed to create Set-HpcClusterName.ps1"
                }

                $process = Start-Process -FilePath "PowerShell.exe" -Argument "-ExecutionPolicy Bypass -Command `"& '$SetClusterScriptFile' $clusName`"" -PassThru -Wait
                if($process.ExitCode -ne 0)
                {
                    throw "Failed to update HPC cluster connection string: $($process.ExitCode)"
                }

            } -ArgumentList @($TargetCluster, $scriptContent) -AsJob -JobName $nodeName

            $leftCapacity--
        }
    }

    if($FailedNodes.Count -gt 0)
    {
        Write-Error "The following nodes were not moved to $TargetCluster :`n $FailedNodes" 
    }
}
