﻿<#
.Synopsis
    Specifies the HPC cluster to which the current computer will connect.

.DESCRIPTION
    This script sets the connection string of the HPC cluster to which the current computer (HPC cluster node) will connect.

.NOTES
    This cmdlet requires that the current computer is a compute node, broker node, or workstation node in an HPC Pack cluster.

.EXAMPLE
    PS > Set-HpcClusterName.ps1 -ConnectionString NewHN
    Sets the HPC cluster connection string for the current computer as NewHN

.EXAMPLE
    PS > Set-HpcClusterName.ps1 -ConnectionString NewHN1,NewHN2,NewHN3 -Delay 10
    Sets the HPC cluster connection string for the current computer as 'NewHN1,NewHN2,NewHN3' after 10 seconds

.EXAMPLE
    PS > Set-HpcClusterName.ps1 -ConnectionString NewHN -RunAsScheduledTask
    Schedules a task to set the HPC cluster connection string for the current computer as NewHN
#>
Param
(
    # The connection string of the HPC cluster that you want the current computer connected to.
    [Parameter(Mandatory=$true, Position=0)]
    [ValidateNotNullOrEmpty()]
    [String[]] $ConnectionString,

    # If specified, the delay time in seconds for the operation.
    [Parameter(Mandatory=$false)]
    [ValidateRange(0, 3600)]
    [int] $Delay = 0,

    # If specified, sets the HPC cluster connection string using a scheduled task.
    [Parameter(Mandatory=$false)]
    [Switch] $RunAsScheduledTask
)

$VerbosePreference = "Continue"
$datestr = Get-Date -Format "yyyy_MM_dd-HH_mm_ss"
$Script:LogFile = "$env:temp\Set-HpcClusterName-$datestr.log"
$ClusterConnectionString = $ConnectionString -join ','

function WriteLog
{
    Param(
        [Parameter(Mandatory=$true, Position=0)]
        [ValidateNotNullOrEmpty()]
        [String] $Message,

        [Parameter(Mandatory=$false)]
        [ValidateSet("Error","Warning","Verbose")]
        [String] $LogLevel = "Verbose"
    )
    
    $timestr = Get-Date -Format 'MM/dd/yyyy HH:mm:ss'
    $NewMessage = "$timestr - $Message"
    switch($LogLevel)
    {
        "Error"     {Write-Error   $NewMessage; break}
        "Warning"   {Write-Warning $NewMessage; break}
        "Verbose"   {Write-Verbose $NewMessage; break}
    }
       
    try
    {
        $NewMessage = "[$LogLevel]$timestr - $Message"
        Add-Content $Script:LogFile $NewMessage -ErrorAction SilentlyContinue
    }
    catch
    {
        #Ignore the error
    }
}

try
{
    $HPCKeyPath = "HKLM:\SOFTWARE\Microsoft\HPC"
    $HPCWow6432KeyPath = "HKLM:\SOFTWARE\Wow6432Node\Microsoft\HPC"
    $clusNameItem = $null
    $roleItem = $null
    $keyExists = Test-Path -Path $HPCKeyPath
    if($keyExists)
    {
        $clusNameItem = Get-ItemProperty -Name ClusterConnectionString -LiteralPath $HPCKeyPath -ErrorAction SilentlyContinue
        $roleItem = Get-ItemProperty -Name InstalledRole -LiteralPath $HPCKeyPath -ErrorAction SilentlyContinue
    }

    if((-not $keyExists) -or ($clusNameItem -eq $null) -or ($roleItem -eq $null))
    {
        throw "$env:ComputerName is not a valid HPC cluster node"
    }

    if($roleItem.InstalledRole -contains "HN")
    {
        throw "$env:ComputerName cannot be moved to the cluster '$ClusterConnectionString' because it is a head node"
    }

    if($clusNameItem.ClusterConnectionString -eq $ClusterConnectionString)
    {
        WriteLog "$env:ComputerName already belongs to HPC cluster $ClusterConnectionString" -LogLevel Warning
        return
    }

    if($RunAsScheduledTask.IsPresent)
    {
        # Schedule a task to run this script itself, start the scheduled task immediately and return.
        $selfFullPath = $MyInvocation.MyCommand.Definition
        WriteLog "The full path of this script is $selfFullPath" -LogLevel Verbose
        # Because ScheduledTasks PowerShell module not available in Windows Server 2008 R2,
        # We use ComObject Schedule.Service to schedule task
        WriteLog "Scheduling task to set HPC cluster connection string to '$ClusterConnectionString' with a delay of $Delay seconds" -LogLevel Verbose
        $randNum = Get-Random -Maximum 100 -Minimum 1
        $taskName = "SetHpcCluster_$randNum"
        $schdService = new-object -ComObject "Schedule.Service"
        $schdService.Connect("localhost")
        $rootFolder = $schdService.GetFolder("\")
        $taskDefinition = $schdService.NewTask(0)
        $setAction = $taskDefinition.Actions.Create(0)
        $setAction.Path = "PowerShell.exe"
        $setClusNameCmd = ". '$selfFullPath' -ConnectionString $ClusterConnectionString -Delay $Delay"
        $setAction.Arguments = '-ExecutionPolicy ByPass -Command "{0}"' -f $setClusNameCmd
        $removeAction = $taskDefinition.Actions.Create(0)
        $removeAction.Path = "SchTasks.exe"
        $removeAction.Arguments = "/Delete /TN $taskName /F"
        $setClusterTask = $Rootfolder.RegisterTaskDefinition($taskName, $taskDefinition, 2, "system", $null, 5)
        try
        {
            $setClusterTask.Run($null) | Out-Null
        }
        catch
        {
            $Rootfolder.DeleteTask($taskName, 0) | Out-Null
            throw
        }
        WriteLog "The task starts to run to set the HPC cluster connection string" -LogLevel Verbose
        return
    }

    if($Delay -gt 0)
    {
        WriteLog "HPC cluster connection string will be updated after $Delay seconds" -LogLevel Verbose
        Start-Sleep -Seconds $Delay
    }

    WriteLog "The current Cluster connection string: $($clusNameItem.ClusterConnectionString)" -LogLevel Verbose
    WriteLog "The current Installed HPC Role(s): $($roleItem.InstalledRole)" -LogLevel Verbose

    # Check the HPC Version by the production version of HpcNodeManager.exe
    $HpcNodeManagerExePath = [System.IO.Path]::Combine($env:CCP_HOME, "Bin\HpcNodeManager.exe")
    if(-not (Test-Path -Path $HpcNodeManagerExePath -PathType Leaf))
    {
        throw "$env:ComputerName is not a valid HPC cluster node: $HpcNodeManagerExePath not found"
    }

    $hpcVersion = [System.Diagnostics.FileVersionInfo]::GetVersionInfo($HpcNodeManagerExePath).ProductVersion
    WriteLog "The current HPC production version is $hpcVersion" -LogLevel Verbose
    $BNServiceList = @("HpcBroker")
    if($hpcVersion -ge "4.0")
    {
        $CNServiceList = @("HpcManagement", "HpcNodeManager", "msmpi", "HpcMonitoringClient", "HpcSoaDiagMon")
        $WNServiceList = @("HpcManagement", "HpcNodeManager", "msmpi", "HpcMonitoringClient")
    }
    else
    {
        $CNServiceList = $WNServiceList = @("HpcManagement", "HpcNodeManager", "msmpi")
    }

    $hpcServices = @()
    if($roleItem.InstalledRole -contains "BN")
    {
        $hpcServices += $BNServiceList
    }
    if($roleItem.InstalledRole -contains "CN")
    {
        $hpcServices += $CNServiceList
    }
    if($roleItem.InstalledRole -contains "WN")
    {
        $hpcServices += $WNServiceList
    }

    $hpcServices = $hpcServices | Select -Unique
    if($hpcServices.Count -eq 0)
    {
        throw "$env:ComputerName is not a valid HPC cluster node"
    }

    # Check the existence of the HPC Services and stop them
    foreach($svcname in $hpcServices)
    {
        $service = Get-Service -Name $svcname -ErrorAction SilentlyContinue
        if($service -eq $null)
        {
            throw "The service $svcname doesn't exist"
        }
        else
        {
            WriteLog "Stopping service: $svcname" -LogLevel Verbose
            Stop-Service -Name $svcname -Force
        }
    }

    WriteLog "Updating HPC cluster connection string in Registry Table on $env:ComputerName" -LogLevel Verbose
    Set-ItemProperty -Path $HPCKeyPath -Name ClusterConnectionString -Value $ClusterConnectionString
    if(Test-Path $HPCWow6432KeyPath)
    {
        Set-ItemProperty -Path $HPCWow6432KeyPath -Name ClusterConnectionString -Value $ClusterConnectionString
    }

    WriteLog "Updating environment variable 'CCP_SCHEDULER' on $env:ComputerName" -LogLevel Verbose
    [Environment]::SetEnvironmentVariable("CCP_SCHEDULER", $ClusterConnectionString, [System.EnvironmentVariableTarget]::Machine)

    WriteLog "Updating environment variable 'CCP_CONNECTIONSTRING' on $env:ComputerName" -LogLevel Verbose
    [Environment]::SetEnvironmentVariable("CCP_CONNECTIONSTRING", $ClusterConnectionString, [System.EnvironmentVariableTarget]::Machine)

    # Set the Startup Type of HPC Services to 'Automatic' in case some of them are not 'Automatic'
    $hpcServices | Set-Service -StartupType Automatic
    $startFailure = $false
    foreach($svcname in $hpcServices)
    {
        Start-Service -Name $svcname
        if(-not $?)
        {
            $startFailure = $true
            WriteLog ("Failed to start service: $svcname : " + $Error[0]) -LogLevel Warning
        }
    }

    if(-not $startFailure)
    {
        WriteLog "Successfully updated HPC cluster connection string to $ClusterConnectionString" -LogLevel Verbose
    }
    else
    {
        # if failed to restart service, restart the computer as a workaround
        Restart-Computer -Force -Delay 20
    }
}
catch
{
    WriteLog "Failed to update HPC cluster connection string to $ClusterConnectionString : $_" -LogLevel Error
    throw
}
