﻿<#
.Synopsis
   Upgrade the HPC application on the Service Fabric cluster
.DESCRIPTION
   This script upgrades the HPC application on the Service Fabric cluster, the script must be run on HPC head node.
   Author :  Microsoft HPC Pack team
   Version:  1.1
.EXAMPLE
   .\Upgrade-HpcApplication.ps1 -ApplicationPackagePath "d:\HpcApplicationType.sfpkg"

.INPUTS
   None
.OUTPUTS
   None
#>
Param
(
    # The file path of HPC application pacakge. If not specified, the script will search HpcApplicationType.sfpkg in the current folder.
    [Parameter(Mandatory=$false)]
    [String] $ApplicationPackagePath
)

Add-PSSnapin Microsoft.Hpc

# If ApplicationPackagePath not specified, use HpcApplicationType.sfpkg in the current folder
if (-not $ApplicationPackagePath)
{
    $ApplicationPackagePath = [IO.Path]::Combine($PSScriptRoot, "HpcApplicationType.sfpkg")
}

if (!(Test-Path $ApplicationPackagePath -PathType Leaf))
{
    throw "The HPC application package file $ApplicationPackagePath doesn't exist"
}

function ExecuteSqlScript([String]$ConnectionString, [String]$Script, [Switch]$File)
{
    $sqlConnection = New-Object -TypeName System.Data.SqlClient.SqlConnection -ArgumentList $ConnectionString
    $sqlConnection.Open()
    $sqlCmd = $sqlConnection.CreateCommand()
    if($File)
    {
        $Script = [IO.File]::ReadAllText($Script)
    }

    $regexOpt = [System.Text.RegularExpressions.RegexOptions]::Compiled -bor [System.Text.RegularExpressions.RegexOptions]::IgnoreCase -bor [System.Text.RegularExpressions.RegexOptions]::Multiline
    [System.Text.RegularExpressions.Regex]::Split($Script, '^\s*GO\s*$', $regexOpt) | ?{-not [string]::IsNullOrWhiteSpace($_)} | %{$sqlCmd.CommandText = $_; $sqlCmd.ExecuteNonQuery() | Out-Null}
    $sqlCmd.Dispose();
    $sqlConnection.Dispose();
}

$AppPkgPathToUse = [IO.Path]::combine($env:Temp, (Get-Item $ApplicationPackagePath).BaseName)
if (Test-Path $AppPkgPathToUse)
{
    Remove-Item -Path $AppPkgPathToUse -Recurse -ErrorAction Stop | Out-Null
}

New-Item $AppPkgPathToUse -ItemType directory | Out-Null
try 
{
    Write-Host "Unpacking the HPC application package ..."
    [System.Reflection.Assembly]::LoadWithPartialName("System.IO.Compression.FileSystem") | Out-Null 
    [System.IO.Compression.ZipFile]::ExtractToDirectory($ApplicationPackagePath, $AppPkgPathToUse) 
} 
catch 
{ 
    throw "Unexpected Error to unzip $ApplicationPackagePath. Error details: $_.Exception.Message" 
}

$ApplicationManifestPath = "$AppPkgPathToUse\ApplicationManifest.xml"
if (!(Test-Path $ApplicationManifestPath))
{
    throw "$ApplicationManifestPath is not found, service fabric application package is invalid"
}

$manifestXml = [xml] (Get-Content $ApplicationManifestPath)
$applicationTypeName = $manifestXml.ApplicationManifest.ApplicationTypeName
$applicationTypeVersion = $manifestXml.ApplicationManifest.ApplicationTypeVersion
if($applicationTypeName -ne 'HpcApplicationType')
{
    throw "Not an HpcApplication package"
}


Connect-ServiceFabricCluster | Out-Null
try
{
    [void](Test-ServiceFabricClusterConnection)
}
catch
{
    Write-Host "Connecting to the Service Fabric Cluster ..."
    Connect-ServiceFabricCluster | Out-Null
}

try
{
    [void](Test-ServiceFabricClusterConnection)
}
catch
{
    Write-Warning "No connection to Service Fabric cluster."
    throw
}

## Check upgrade status
$hpcApplication = Get-ServiceFabricApplication -ApplicationName fabric:/HpcApplication
if(!$hpcApplication)
{
    throw "fabric:/HpcApplication not found"
}

if ($hpcApplication.ApplicationTypeVersion -eq $applicationTypeVersion)
{
    Write-Warning "The application version of fabric:/HpcApplication is already $applicationTypeVersion."
    return
}

$appParameters = @{}
foreach($appParam in $hpcApplication.ApplicationParameters)
{
    $appParameters[$appParam.Name] = $appParam.Value
}

$upgradeStatus = Get-ServiceFabricApplicationUpgrade -ApplicationName fabric:/HpcApplication
if ($upgradeStatus.UpgradeState -ne "RollingBackCompleted" -and $upgradeStatus.UpgradeState -ne "RollingForwardCompleted")
{
    $errMsg = "An upgrade for the application '$names.ApplicationTypeName' is already in progress."
    throw $errMsg
}

$hpcApplication = Get-ServiceFabricApplication -ApplicationName fabric:/HpcApplication
if ($hpcApplication.ApplicationTypeVersion -eq $applicationTypeVersion)
{
    Write-Warning "The current version of fabric:/HpcApplication is already $applicationTypeVersion."
    return
}

## Upgrade DB
Write-Host "Updating HPC Scheduler database..."
$schdDbConn = (Get-HpcClusterRegistry -PropertyName SchedulerDbConnectionString).Value
$schdscript = "DECLARE @dbname varchar(100)
SET @dbname=QUOTENAME(DB_NAME())
if SERVERPROPERTY('EngineEdition') < 5
BEGIN
  EXEC('ALTER DATABASE '+@dbname+' SET SINGLE_USER WITH ROLLBACK IMMEDIATE')
  EXEC('ALTER DATABASE '+@dbname+' SET READ_COMMITTED_SNAPSHOT ON')
  EXEC('ALTER DATABASE '+@dbname+' SET MULTI_USER')
END"
ExecuteSqlScript -ConnectionString $schdDbConn -Script $schdscript

## Upgrade Application
$reg = Get-ServiceFabricApplicationType -ApplicationTypeName $applicationTypeName | ?{$_.ApplicationTypeVersion -eq $applicationTypeVersion}
if ($reg)
{
    Write-Host "Application Type '$applicationTypeName' with Version '$applicationTypeVersion' was already registered with cluster"
}
else
{
    Write-Host "Copying application package to image store..."
    $applicationPackagePathInImageStore = "$applicationTypeName\$applicationTypeVersion"
    Copy-ServiceFabricApplicationPackage -ApplicationPackagePath $AppPkgPathToUse -ImageStoreConnectionString fabric:ImageStore -ApplicationPackagePathInImageStore $applicationPackagePathInImageStore
    if(!$?)
    {
        throw "Copying of application package to image store failed. Cannot continue with registering the application."
    }
    
    Write-Host "Registering application type..."
    Register-ServiceFabricApplicationType -ApplicationPathInImageStore $applicationPackagePathInImageStore
    Write-Host "Removing application package from image store ..."
    Remove-ServiceFabricApplicationPackage -ApplicationPackagePathInImageStore $applicationPackagePathInImageStore -ImageStoreConnectionString fabric:ImageStore
}

Write-Host "Start upgrading application..." 
Start-ServiceFabricApplicationUpgrade -ApplicationName fabric:/HpcApplication -ApplicationTypeVersion $applicationTypeVersion -ApplicationParameter $appParameters `
    -HealthCheckStableDurationSec 60 -UpgradeDomainTimeoutSec 1800 -UpgradeTimeout 3000 -FailureAction Rollback -Monitored | Out-Null

Write-Host "Waiting for upgrade..."
do
{
    Write-Host '.' -NoNewline    
    Start-Sleep -Seconds 3
    $upgradeStatus = Get-ServiceFabricApplicationUpgrade -ApplicationName fabric:/HpcApplication
} while ($upgradeStatus.UpgradeState -ne "RollingBackCompleted" -and $upgradeStatus.UpgradeState -ne "RollingForwardCompleted")

Write-Host
if($upgradeStatus.UpgradeState -eq "RollingForwardCompleted")
{
    Write-Host "fabric:/HpcApplication upgrading completed successfully."
}
elseif($upgradeStatus.UpgradeState -eq "RollingBackCompleted")
{
    Write-Host "fabric:/HpcApplication upgrading failed and was rolled back."
}
